from typing import Union


class Vector3f:
    def __init__(self, x: float, y: float, z: float):
        self.x = x
        self.y = y
        self.z = z

    def __add__(self, other: "Vector3f"):
        return Vector3f(self.x + other.x, self.y + other.y, self.z + other.z)

    def __sub__(self, other: "Vector3f"):
        return Vector3f(self.x - other.x, self.y - other.y, self.z - other.z)

    def __mul__(self, other: Union[float, int]):
        if isinstance(other, Vector3f):
            raise NotImplementedError("To multiply two vectors, use a @ b to compute the dot product")
        return Vector3f(other * self.x, other * self.y, other * self.z)

    def __rmul__(self, other: Union[float, int]):
        return self * other

    def __matmul__(self, other: "Vector3f") -> float:
        """Dot product"""
        return self.x * other.x + self.y * other.y + self.z * other.z

    def __neg__(self):
        return Vector3f(-self.x, -self.y, -self.z)

    def __repr__(self):
        return f"Vector3f({self.x}, {self.y}, {self.z})"


if __name__ == "__main__":
    v = Vector3f(1, 2, 3)
    k = 3 * v
    print(v)
    print(k)
    print(v * 6)
    print(v @ k)



# This file includes various functions which use any of the various windows dlls to determine things.

import ctypes
from ctypes import wintypes

import pymem

GetModuleFileNameExA = ctypes.windll.psapi.GetModuleFileNameExA
GetModuleFileNameExA.restype = wintypes.DWORD
GetModuleFileNameExA.argtypes = [
    wintypes.HANDLE,
    wintypes.HMODULE,
    wintypes.LPSTR,
    wintypes.DWORD,
]

GetWindowLongA = ctypes.windll.user32.GetWindowLongA
GetWindowLongA.restype = wintypes.LONG
GetWindowLongA.argtypes = [
    wintypes.HWND,
    ctypes.c_int,
]

SetWindowLongA = ctypes.windll.user32.SetWindowLongA
SetWindowLongA.restype = wintypes.LONG
SetWindowLongA.argtypes = [
    wintypes.HWND,
    ctypes.c_int,
    wintypes.LONG,
]

SetLayeredWindowAttributes = ctypes.windll.user32.SetLayeredWindowAttributes
SetLayeredWindowAttributes.restype = wintypes.BOOL
SetLayeredWindowAttributes.argtypes = [
    wintypes.HWND,
    wintypes.COLORREF,
    wintypes.BYTE,
    wintypes.DWORD,
]

MAX_EXE_NAME_SIZE = 1024
WS_EX_LAYERED = 0x00080000  # layered window
GWL_EXSTYLE = -20  # "extended window style"

LWA_COLORKEY = 0x00000001
LWA_ALPHA = 0x00000002


def get_exe_path_from_pid(proc: pymem.Pymem) -> str:
    """Get the name of the exe which was run to create the pymem process."""
    name_buffer = ctypes.create_string_buffer(b"", MAX_EXE_NAME_SIZE)
    GetModuleFileNameExA(proc.process_handle, None, name_buffer, MAX_EXE_NAME_SIZE)
    return name_buffer.value.decode()


def set_window_transparency(hwnd: int, alpha: float):
    SetWindowLongA(hwnd, GWL_EXSTYLE, GetWindowLongA(hwnd, GWL_EXSTYLE) | WS_EX_LAYERED)
    rgb = wintypes.RGB(10, 10, 10)
    SetLayeredWindowAttributes(hwnd, rgb, int(255 * alpha), LWA_ALPHA)



from collections import namedtuple
from enum import Enum
from typing import Any, Optional, Protocol

FUNCDEF = namedtuple("FUNCDEF", ["restype", "argtypes"])


class DetourTime(Enum):
    NONE = 0
    BEFORE = 1
    AFTER = 2


class KeyPressProtocol(Protocol):
    _hotkey: str
    _hotkey_press: str


class HookProtocol(Protocol):
    _is_funchook: bool
    _is_manual_hook: bool
    _is_imported_func_hook: bool
    _has__result_: bool
    _hook_func_name: str
    _hook_time: DetourTime
    _custom_trigger: Optional[str]
    _func_overload: Optional[str]

    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...


class ManualHookProtocol(HookProtocol):
    _hook_offset: Optional[int]
    _hook_pattern: Optional[str]
    _hook_binary: Optional[str]
    _hook_func_def: FUNCDEF


class ImportedHookProtocol(HookProtocol):
    _dll_name: str
    _hook_func_def: FUNCDEF



FNV_offset_basis = 0xCBF29CE484222325
FNV_prime = 0x100000001B3


def fnv_1a(input: str, length: int):
    _input = input.ljust(length, "\x00")
    _hash = FNV_offset_basis
    for char in _input:
        _hash = (ord(char) ^ _hash) & 0xFFFFFFFFFFFFFFFF
        _hash = (_hash * FNV_prime) & 0xFFFFFFFFFFFFFFFF
    return _hash
